import pypm
import thread
import threading
import wx
import time, math, os
import socket
if os.name == 'nt':
	import win32api
	import win32con
	import win32gui
    

buttonMappings = {
    "u":60,
    "l":61,
    "d":62,
    "r":63,
    "x":64,
    "o":65,
    "s":66,
    "t":67
    }

chordButtonMappings = {
    "s":3,
    "x":0,
    "o":1,
    "t":2
    
}    
    
    

chord = {
    "maj":(1,5,8,13),
    "m":(1,4,8,13),
    "sus2":(1,3,8,13),
    "sus4":(1,6,8,13),
    "majb5":(1,5, 7,13),
    "dim":(1, 4, 7,13),
    "aug":(1, 5, 9,13),
    "7":(1, 5, 8, 11)
    }
    
    
    

class PSPInput(threading.Thread):
    def __init__(self, parent):
        threading.Thread.__init__(self)
        self.parent = parent
        self.running = True
             
        self.serversocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.serversocket.bind((socket.gethostname(), 2006))
        self.serversocket.settimeout(1)
        self.serversocket.listen(1)
        
        
    def stop(self):    
        self.running = False
    def run(self):
        while self.running:
            print "Waiting for connections on port 2006"
            clientSocket = None
            while self.running:
                try:
                    (clientSocket, address) = self.serversocket.accept()
                    self.parent.PSPChannel[address[0]] = 0
                    break
                except socket.timeout:
                    pass
            if clientSocket:
                
                print "Client Connected! Starting thread"
                print address
                clientSocket.setblocking(True)
                clientThread = threading.Timer(0.01, self.getPSPData, (clientSocket, address))
                clientThread.start()
            


    def getPSPData(self, clientSocket, address):
        try:
            data = clientSocket.recv(1)
        except:
            data = None

        if data == "a":

                analogData = (int(clientSocket.recv(3))-500,int(clientSocket.recv(3))-500, address)
                self.parent.pspAnalogBuffer.append(analogData)

            
            
            #print analogData
        elif data:
            state = None
            while not state:
                try:
                    state = clientSocket.recv(1)
                except:
                    pass
                
            self.parent.pspButtonBuffer.append((data, state, address))
        try:
            clientSocket.send("Channel:" + str(self.parent.PSPChannel[address[0]]))
        except:
            print "Client Disconnected"
            del(self.parent.PSPChannel[address[0]])
        else:
            if self.running:
                clientThread = threading.Timer(0, self.getPSPData, (clientSocket,address))
                clientThread.start()

class PSP2MIDI(wx.App):
    def __init__(self, size = (230, 150)):
        self.size = size
        wx.App.__init__(self, False)

        self.currentChordType = "maj"

        self.x = 64
        self.y = 64
        self.angle = 0
        self.baseNote = 0
        self.octave = 4
        
        self.oldx = 0
        self.oldy = 0
        self.lShoulder = False
        self.rShoulder = False
        self.mode = "chord"

        self.buttonStates = {"u":0,"l":0,"d":0,"r":0,"x":0,"o":0,"s":0,"t":0,"q":0}
        self.pspButtonBuffer = []
        self.pspAnalogBuffer = []
        self.PSPChannel = {}
        
        self.pspServer = PSPInput(self)
        self.pspServer.start()

        
        
        
    def OnInit(self):
        print "Initializing UI"
        pypm.Initialize()
        
        
        self.stopping = False

#        self.midiIn = None        
#        self.devices = self.getInputDevices()
#        self.currentDevice = self.devices.keys()[0]
        self.midiOut = None
        self.outputDevices = self.getOutputDevices()
        self.currentOutputDevice = self.outputDevices.keys()[0]


        
        self.frame = wx.Frame(None, -1, "PSP2Midi", size=self.size)
        self.SetTopWindow(self.frame)
 #       wx.StaticText(self.frame, -1, "Midi Inputs", (8, 10))
 #       self.deviceUI = wx.ComboBox(self.frame, -1, self.currentDevice, (10, 30), (200, -1), choices = self.devices.keys())
 #       self.Bind(wx.EVT_COMBOBOX, self.switchDevice, self.deviceUI)
        wx.StaticText(self.frame, -1, "Local Output", (8, 60))
        self.outputDeviceUI = wx.ComboBox(self.frame, -1, str(self.currentOutputDevice), (10, 80), (200, -1), choices = self.outputDevices.keys())
        self.Bind(wx.EVT_COMBOBOX, self.switchOutputDevice, self.outputDeviceUI)

        self.frame.Show(True)

        
        return True
    def start(self):
        thread.start_new(self.midiLoop, ())
        self.MainLoop()
    def OnExit(self):
        self.stopping = True
        del self.midiOut
        #del self.midiIn

        
        pypm.Terminate()
        self.pspServer.stop()
        self.ExitMainLoop()
                

    def switchDevice(self, evt):
        self.currentDevice = self.deviceUI.GetValue()
        self.changeDevice()
        
    def switchOutputDevice(self, evt):
        self.currentOutputDevice = self.outputDeviceUI.GetValue()        
        self.changeOutputDevice()
        
    def getInputDevices(self):
        inputDevices = {}
        for deviceNumber in range(pypm.CountDevices()):
            interf,name,inp,outp,opened = pypm.GetDeviceInfo(deviceNumber)
            print interf, name, inp, outp, opened
            if inp:
                inputDevices[name] = deviceNumber
        return inputDevices
    def getOutputDevices(self):
        outputDevices = {}
        for deviceNumber in range(pypm.CountDevices()):
            interf,name,inp,outp,opened = pypm.GetDeviceInfo(deviceNumber)
            print interf, name, inp, outp, opened
            if outp:
                outputDevices[name] = deviceNumber
        return outputDevices   



    def changeDevice(self):
        del self.midiIn
        self.midiIn = pypm.Input(self.devices[self.currentDevice])
    def changeOutputDevice(self): 
        del self.midiOut
        self.midiOut = pypm.Output(self.outputDevices[self.currentOutputDevice], 0)
    def midiLoop(self):
        #self.changeDevice()

        self.changeOutputDevice()
        while 1:
            if self.stopping:
                return 1


            

            if self.mode == "note":
                if self.pspAnalogBuffer:
                    while len(self.pspAnalogBuffer):
                        x, y, address = self.pspAnalogBuffer.pop()
                        self.oldx = self.x
                        self.oldy = self.y
                        self.x = (x + 128)/2
                        self.y = (y + 128)/2
                        if self.oldx != self.x:
                            self.midiOut.WriteShort(0xE0 + self.PSPChannel[address[0]], 0, self.x)
                        if self.oldy != self.y:
                            self.midiOut.WriteShort(0xB0 + self.PSPChannel[address[0]], 0, self.y)
           

                if self.pspButtonBuffer:
                    while len(self.pspButtonBuffer):
                        button, state, address = self.pspButtonBuffer.pop()
                        state = int(state)
                        self.buttonStates[button] = state
                        print button, state, address
                        
                        
                        if self.buttonStates["q"]:
                            if button == "i" and state:
                            	if os.name == "nt":
                                	self.mode = "mouse"
                                else:
                                	self.mode = "chord"
                            elif button == "e" and state:
                                self.mode = "chord"
                        else:
                            if button == "e":
                                if state:
                                    self.lShoulder = True
                                else:
                                    self.lShoulder = False
                            elif button == "i":
                                if state:
                                    self.rShoulder = True
                                else:
                                    self.rShoulder = False
                            if button in buttonMappings.keys():
                                note = buttonMappings[button]
                                if self.lShoulder:
                                    note = note + 8
                                if self.rShoulder:
                                    note = note + 16
                                if state:
                                    self.midiOut.WriteShort(0x90 + self.PSPChannel[address[0]], note, 100)
                                else:
                                    if not self.rShoulder and not self.lShoulder:
                                        self.midiOut.WriteShort(0x90 + self.PSPChannel[address[0]], note, 0)
                            
            elif self.mode == "chord":
                if self.pspAnalogBuffer:
                    while len(self.pspAnalogBuffer):
                        x, y, address = self.pspAnalogBuffer.pop()
                        
                        dist = math.hypot(x, y)
                        if dist > 100:
                            self.angle = math.degrees(math.atan2(-y, x))
                            if self.angle < 0:
                                self.angle = 360+self.angle
                            self.baseNote = int(math.floor(self.angle/30))
                if self.pspButtonBuffer:
                    while len(self.pspButtonBuffer):
                        button, state, address = self.pspButtonBuffer.pop()
                        
                        state = int(state)
                        self.buttonStates[button] = state


                        if self.buttonStates["q"]:
                            if button == "i" and state:
                                self.mode = "note"
                            elif button == "e" and state:
                            	if os.name == "nt":
                                	self.mode = "mouse"
                                else:
                                	self.mode = "note"
                                	
                            elif button == "l" and state:
                                self.PSPChannel[address[0]] = self.PSPChannel[address[0]] - 1
                            elif button == "r" and state:
                                self.PSPChannel[address[0]] = self.PSPChannel[address[0]] + 1
                            elif button == "u" and state:
                                self.midiOut.WriteShort(0xc0 + self.PSPChannel[address[0]], 100)
                            

                            if self.PSPChannel[address[0]] == 16:
                                self.PSPChannel[address[0]] = 0
                            if self.PSPChannel[address[0]] == -1:
                                self.PSPChannel[address[0]] = 15
                                
                        else:                                
                            if button in chordButtonMappings.keys():
                                note = self.octave * 12 + chord[self.currentChordType][chordButtonMappings[button]] + self.baseNote
                                if state:
                                    self.midiOut.WriteShort(0x90 + self.PSPChannel[address[0]], note, 100)
                                else:
                                    self.midiOut.WriteShort(0x90 + self.PSPChannel[address[0]], note, 0)
                            
                                
                            if self.buttonStates["u"] and self.buttonStates["l"]:
                                self.currentChordType = "majb5"
                            elif self.buttonStates["u"] and self.buttonStates["r"]:
                                self.currentChordType = "7"
                            elif self.buttonStates["d"] and self.buttonStates["l"]:
                                self.currentChordType = "sus4"
                            elif self.buttonStates["d"] and self.buttonStates["r"]:
                                self.currentChordType = "sus2"
                            elif self.buttonStates["u"]:
                                self.currentChordType = "maj"
                            elif self.buttonStates["d"]:
                                self.currentChordType = "m"
                            elif self.buttonStates["l"]:
                                self.currentChordType = "dim"
                            elif self.buttonStates["r"]:
                                self.currentChordType = "aug"
                            
                            if button == "e" and state and self.octave:
                                self.octave = self.octave - 1
                            if button == "i" and state and self.octave < 10:
                                self.octave = self.octave + 1
            elif self.mode == "mouse":
 
                if self.pspAnalogBuffer:
                    while len(self.pspAnalogBuffer):
                        x, y, address = self.pspAnalogBuffer.pop()
                        self.x = x
                        self.y = y
                        if self.x > 25 or self.x < -25 or self.y > 25 or self.y < -25:              
                            win32api.mouse_event(win32con.MOUSEEVENTF_MOVE, self.x/32, self.y/32, 0, 0)
                    
           

                if self.pspButtonBuffer:
                    while len(self.pspButtonBuffer):
                        button, state, address = self.pspButtonBuffer.pop()
                        state = int(state)
                        self.buttonStates[button] = state
                        print button, state, address

                        if self.buttonStates["q"]:
                            if button == "i" and state:
                                self.mode = "chord"
                            elif button == "e" and state:
                                self.mode = "note"

                        else:
                            if button == "x":
                                if state:
                                    win32api.mouse_event(win32con.MOUSEEVENTF_LEFTDOWN, 0, 0, 0, 0)
                                else:
                                    win32api.mouse_event(win32con.MOUSEEVENTF_LEFTUP, 0, 0, 0, 0)
                            if button == "o":
                                if state:
                                    win32api.mouse_event(win32con.MOUSEEVENTF_RIGHTDOWN, 0, 0, 0, 0)
                                else:
                                    win32api.mouse_event(win32con.MOUSEEVENTF_RIGHTUP, 0, 0, 0, 0)       

            time.sleep(0.001)
            
            
        return 1

    









app = PSP2MIDI()
app.start()